@@ -32,7 +32,6 @@ __all__ = [
     "ALAddendumFieldDict",
     "ALDocumentBundle",
     "ALDocument",
-    "ALDocumentBundleDict",
     "ALStaticDocument",
     "safeattr",
     "label",
@@ -43,21 +42,28 @@ __all__ = [
     "ALTableDocument",
     "ALUntransformedDocument",
     "unpack_dafilelist",
+    "ALDocumentUpload",
 ]
 
 DEBUG_MODE = get_config("debug")
 
 
 def label(dictionary):
+    """Given a dictionary like: {"some_attribute":"Some label"}, return the `value` of the first dictionary item.
+    Useful for working with the `columns` method of an ALAddendumField.
+    """
     try:
-        return list(dictionary.items())[0][1]
+        return next(iter(dictionary.values()), "")
     except:
         return ""
 
 
 def key(dictionary):
+    """Given a dictionary like: {"some_attribute":"Some label"}, return the `key` of the first dictionary item.
+    Useful for working with the `columns` method of an ALAddendumField.
+    """
     try:
-        return list(dictionary.items())[0][1]
+        return next(iter(dictionary.keys()), "")
     except:
         return ""
 
@@ -599,7 +605,9 @@ class ALDocument(DADict):
         ---
         question: |
           Do you want the extra document included?
-        yesno: include_extra_document
+        fields:
+          - no label: include_extra_document
+            datatype: yesnoradio
         ---
         code: |
           extra_document.enabled = include_extra_document
@@ -1318,42 +1326,6 @@ class ALDocumentBundle(DAList):
             )
 
 
-class ALDocumentBundleDict(DADict):
-    """
-    A dictionary with named bundles of ALDocuments. In the assembly line, we
-    expect to find two predetermined bundles: court_bundle and user_bundle.
-
-    It may be helpful in some circumstances to have a "bundle" of bundles. E.g.,
-    you may want to present the user multiple combinations of documents for
-    different scenarios.
-    """
-
-    def init(self, *pargs, **kwargs):
-        super().init(*pargs, **kwargs)
-        self.auto_gather = False
-        self.gathered = True
-        self.object_type = ALDocumentBundle
-        if not hasattr(self, "gathered"):
-            self.gathered = True
-        if not hasattr(self, "auto_gather"):
-            self.auto_gather = False
-
-    def preview(self, format: str = "PDF", bundle: str = "user_bundle") -> DAFile:
-        """
-        Create a copy of the document as a single PDF that is suitable for a preview version of the
-        document (before signature is added).
-        """
-        return self[bundle].as_pdf(key="preview", format=format)
-
-    def as_attachment(
-        self, format: str = "PDF", bundle: str = "court_bundle"
-    ) -> List[DAFile]:
-        """
-        Return a list of PDF-ified documents, suitable to make an attachment to send_mail.
-        """
-        return self[bundle].as_pdf_list(key="final")
-
-
 class ALExhibit(DAObject):
     """Class to represent a single exhibit, with cover page, which may contain multiple documents representing pages.
     Atributes:
@@ -1484,7 +1456,7 @@ class ALExhibitList(DAList):
             auto_labeler = self.auto_labeler
 
         for index, exhibit in enumerate(self.elements):
-            exhibit.label = self.auto_labeler(index)
+            exhibit.label = auto_labeler(index)
 
     def ocr_ready(self) -> bool:
         """
@@ -1549,6 +1521,7 @@ class ALExhibitDocument(ALDocument):
         include_exhibit_cover_pages (bool): flag to control whether cover pages are included with each separate exhibit
         add_page_numbers (bool): Flag that controls whether the as_pdf() method
           also will add Bates-stamp style page numbers and labels on each page.
+        auto_labeler (callable): a Lambda or Python function that will be used to label exhibits.
 
     Todo:
         * Method of making a safe link in place of the attachment (e.g., filesize limits on email)
@@ -1567,11 +1540,20 @@ class ALExhibitDocument(ALDocument):
     objects:
       - al_user_bundle: ALDocumentBundle.using(elements=[my_instructions, my_main_attachment, exhibit_attachment], filename="user_bundle.pdf", title="All forms to download for your records")
     ```
+
+    Example of using a custom label function, https://docassemble.org/docs/functions.html#item_label:
+    ```
+    ---
+    objects:
+      - exhibit_attachment: ALExhibitDocument.using(title="Exhibits", filename="exhibits" , auto_labeler=item_label)
+    ```
     """
 
     def init(self, *pargs, **kwargs):
         super().init(*pargs, **kwargs)
         self.initializeAttribute("exhibits", ALExhibitList)
+        if hasattr(self, "auto_labeler"):
+            self.exhibits.auto_labeler = self.auto_labeler
         if not hasattr(self, "include_table_of_contents"):
             self.include_table_of_contents = True
         if not hasattr(self, "include_exhibit_cover_pages"):
@@ -1679,7 +1661,22 @@ class ALUntransformedDocument(ALDocument):
         return self[key]
 
     def as_docx(self, key: str = "final", refresh: bool = True, **kwargs) -> DAFile:
-        return self.as_pdf(key, refresh, **kwargs)
+        return self[key]
+
+
+class ALDocumentUpload(ALUntransformedDocument):
+    """
+    Simplified class to handle uploaded documents, without any of the complexity of the
+    ALExhibitDocument class.
+    """
+
+    def __getitem__(self, key):
+        # This overrides the .get() method so that the 'final' and 'private' key always exist and
+        # point to the same file.
+        # There's no need to have final/preview versions of an uploaded document
+        if isinstance(self.file, DAFileList):
+            self.file = unpack_dafilelist(self.file)
+        return self.file
 
 
 def unpack_dafilelist(the_file: DAFileList) -> DAFile:
