@@ -20,8 +20,11 @@ from docassemble.base.util import (
     DADateTime,
     subdivision_type,
     this_thread,
+    ensure_definition,
+    DAFile,
 )
 import re
+import pycountry
 
 __all__ = [
     "ALAddress",
@@ -46,6 +49,7 @@ __all__ = [
     "github_modified_date",
     "will_send_to_real_court",
     "safe_subdivision_type",
+    "language_name",
 ]
 
 ##########################################################
@@ -369,7 +373,7 @@ class ALIndividual(Individual):
         if not hasattr(self, "mailing_address"):
             self.initializeAttribute("mailing_address", ALAddress)
 
-    def signature_if_final(self, i: str):
+    def signature_if_final(self, i: str) -> Union[DAFile, str]:
         if i == "final":
             return self.signature
         else:
@@ -438,10 +442,10 @@ class ALIndividual(Individual):
 
     # This design helps us translate the prompts for common fields just once
     def name_fields(
-        self, person_or_business: str = "person", show_suffix=True
+        self, person_or_business: str = "person", show_suffix: bool = True
     ) -> List[Dict[str, str]]:
         """
-        Return suitable field prompts for a name. If `uses_parts` is None, adds the
+        Return suitable field prompts for a name. If `person_or_business` is None, adds the
         proper "show ifs" and uses both the parts and the single entry
         """
         if person_or_business == "person":
@@ -593,6 +597,48 @@ class ALIndividual(Individual):
                 self_described_input,
             ]
 
+    def language_fields(
+        self,
+        choices: List[Dict[str, str]] = [
+            {"English": "en"},
+            {"Spanish": "es"},
+            {"Other": "other"},
+        ],
+        style: str = "radio",
+    ):
+        """Return a standard language picker with an "other" input. Language should be specified as ISO 639-2 or -3 codes so it is compatible with the language_name() function."""
+        other = {
+            "label": str(self.language_other_label),
+            "field": self.attr_name("language_other"),
+            "show if": {"variable": self.attr_name("language"), "is": "other"},
+        }
+        if style == "radio":
+            return [
+                {
+                    "label": str(self.language_label),
+                    "field": self.attr_name("language"),
+                    "input type": "radio",
+                    "choices": choices,
+                },
+                other,
+            ]
+        else:
+            return [
+                {
+                    "label": str(self.language_label),
+                    "field": self.attr_name("language"),
+                    "choices": choices,
+                },
+                other,
+            ]
+
+    def language_name(self):
+        """Return the human-readable version of the individual's language, handling the "other" option."""
+        if self.language == "other":
+            return self.language_other
+        else:
+            return language_name(self.language)
+
     @property
     def gender_male(self):
         """Provide True/False for 'male' gender to assist with checkbox filling
@@ -810,17 +856,24 @@ def is_phone_or_email(text: str) -> bool:
         return True
     else:
         validation_error("Enter a valid phone number or email address")
+        assert False, "unreachable"
 
 
 def github_modified_date(
-    github_user: str, github_repo_name: str
+    github_user: str, github_repo_name: str, auth=None
 ) -> Union[DADateTime, None]:
     """
     Returns the date that the given GitHub repository was modified or None if API call fails.
 
-    Will check for the presence of credentials in the configuration labeled "github readonly"
+    Will check for the presence of credentials in the configuration labeled "github issues"
     in this format:
 
+    github issues:
+      username: YOUR_GITHUB_USERNAME
+      token: YOUR_GITHUB_PRIVATE_TOKEN
+
+    If those credentials aren't found, it will then search for credentials in this format (deprecated):
+
     github readonly:
       username: YOUR_GITHUB_USERNAME
       password: YOUR_GITHUB_PRIVATE_TOKEN
@@ -829,12 +882,37 @@ def github_modified_date(
     If no valid auth information is in the configuration, it will fall back to anonymous authentication.
     The GitHub API is rate-limited to 60 anonymous API queries/hour.
     """
+    if not auth:
+        issue_config = get_config("github issues")
+        if issue_config:
+            auth = {
+                "username": issue_config.get("username"),
+                "password": issue_config.get("token"),
+                "type": "basic",
+            }
+        else:
+            auth = get_config("github readonly")
     github_readonly_web = DAWeb(base_url="https://api.github.com")
     res = github_readonly_web.get(
         "repos/" + github_user + "/" + github_repo_name,
-        auth=get_config("github readonly"),
+        auth=auth,
     )
     if res and res.get("pushed_at"):
         return as_datetime(res.get("pushed_at"))
     else:
         return None
+
+
+# TODO(qs): remove if https://github.com/jhpyle/docassemble/pull/520 is merged
+def language_name(language_code: str) -> str:
+    """Given a 2 digit language code abbreviation, returns the full
+    name of the language. The language name will be passed through the `word()`
+    function."""
+    ensure_definition(language_code)
+    try:
+        if len(language_code) == 2:
+            return word(pycountry.languages.get(alpha_2=language_code).name)
+        else:
+            return word(pycountry.languages.get(alpha_3=language_code).name)
+    except:
+        return language_code
